@@ -2,13 +2,12 @@
 # -*- coding: utf-8 -*-
 # pylint: disable-msg=C0103
 
-###############################################################################
-# Copyright (c) 2006-2016 Franz Inc.
-# All rights reserved. This program and the accompanying materials
-# are made available under the terms of the Eclipse Public License v1.0
-# which accompanies this distribution, and is available at
-# http://www.eclipse.org/legal/epl-v10.html
-###############################################################################
+################################################################################
+# Copyright (c) 2006-2017 Franz Inc.  
+# All rights reserved. This program and the accompanying materials are
+# made available under the terms of the MIT License which accompanies
+# this distribution, and is available at http://opensource.org/licenses/MIT
+################################################################################
 
 from __future__ import absolute_import
 from __future__ import unicode_literals
@@ -18,6 +17,8 @@ from builtins import object
 from future import standard_library
 standard_library.install_aliases()
 
+import os
+
 from ..exceptions import ServerException
 from ..repository.repository import Repository, RepositoryConnection
 from ...miniclient import repository as miniserver
@@ -31,11 +32,15 @@ LEGAL_OPTION_TYPES = {READ_ONLY: bool,}
 
 class AllegroGraphServer(object):
     """
-    Connects to an AllegroGraph HTTP Server
+    The AllegroGraphServer object represents a remote AllegroGraph server on
+    the network. It is used to inventory and access the catalogs of that
+    server.
     """
     def __init__(self, host=None, port=None, user=None, password=None,
                  cainfo=None, sslcert=None, verifyhost=None, verifypeer=None,
-                 protocol=None, proxy=None, **options):
+                 protocol=None,
+                 proxy=os.environ.get('AGRAPH_PROXY'),
+                 **options):
         """
         Define the connection to the AllegroGraph HTTP server.
 
@@ -45,12 +50,14 @@ class AllegroGraphServer(object):
 
         :param host: Address of the AllegroGraph server to connect to.
                      This can also include protocol and port
-                     (e.g. http://localhost:10035).
+                     (e.g. http://localhost:10035). Can also be specified in
+                     the ``AGRAPH_HOST`` environment variable.
                      The default value is ``'localhost'```.
         :type host: string
         :param port: Port on which the server is listening.
                      The default is 10035 if ``protocol`` is ``'http'``
-                     and 10036 if it is ``'https'``.
+                     and 10036 if it is ``'https'``. Can also be specified in
+                     the ``AGRAPH_PORT`` environment variable.
                      If passed explicitly this parameter overrides any value
                      that might have been specified as a part of the ``host`` string.
         :type port: int
@@ -60,9 +67,11 @@ class AllegroGraphServer(object):
                          If passed explicitly this parameter overrides any value
                          that might have been specified as a part of the ``host`` string.
         :type protocol: string
-        :param user: Username (when using Basic authentication).
+        :param user: Username (when using Basic authentication). Can also be specified in
+                     the ``AGRAPH_USER`` environment variable.
         :type user: string
-        :param password: Password (when using Basic authentication).
+        :param password: Password (when using Basic authentication). Can also be specified in
+                         the ``AGRAPH_PASSWORD`` environment variable.
         :type password: string
         :param cainfo: Path to a file or directory containing CA certificates that
                        will be used to validate the server's certificate.
@@ -86,13 +95,15 @@ class AllegroGraphServer(object):
                       Supported schemes are 'http', 'socks4' and 'socks5'.
                       Note that for SOCKS proxies DNS requests are performed by the
                       proxy server.
+                      The default value is taken from the AGRAPH_PROXY environment
+                      variable.
         :type proxy: string
         :param options: Ignored.
         """
         # Not sure why we accept these, but don't want to change the API at this point.
         del options
 
-        host = host or 'localhost'
+        host = host or os.environ.get('AGRAPH_HOST', '127.0.0.1')
 
         # Check if other arguments were passed as a part of host
         match = re.match(r'^(?:(?P<protocol>https?)://)?'
@@ -113,10 +124,16 @@ class AllegroGraphServer(object):
             protocol = 'https' if has_https_params else 'http'
 
         if port is None:
-            port = 10035 if protocol == 'http' else 10036
+            if 'AGRAPH_PORT' in os.environ:
+                port = int(os.environ.get('AGRAPH_PORT'))
+            else:
+                port = 10035 if protocol == 'http' else 10036
 
         uri = '{protocol}://{host}:{port}{tail}'.format(protocol=protocol, host=host, port=port, tail=tail)
 
+        user = user or os.environ.get('AGRAPH_USER')
+        password = password or os.environ.get('AGRAPH_PASSWORD')
+
         self._client = miniserver.Client(uri, user, password, cainfo, sslcert, verifyhost, verifypeer,
                                          proxy=proxy)
 
@@ -131,12 +148,26 @@ class AllegroGraphServer(object):
         return self._client.getVersion()
 
     def listCatalogs(self):
+        """
+        Get the list of catalogs on this server.
+
+        A value of ``None`` will be included in this list to
+        represent the root catalog.
+
+        :return: A list of catalog names.
+        :rtype: list[string]
+        """
         catalogs = self._client.listCatalogs()
         return catalogs
 
     def openCatalog(self, name=None):
         """
         Open a catalog by name. Pass None to open the root catalog.
+
+        :param name: One of the catalog names from :meth:`listCatalogs`
+                     or ``None`` to open the root catalog.
+        :return: A catalog object.
+        :rtype: Catalog
         """
         # Allow for None and '' (or anything else that evaluates to False) to
         # mean the root catalog.
@@ -156,41 +187,86 @@ class AllegroGraphServer(object):
 
         The initialization file is a collection of Common Lisp code
         that is executed in every back-end as it is created.
+        :return: Init file content.
+        :rtype: string
         """
         return self._client.getInitfile()
 
     def setInitfile(self, content=None, restart=True):
         """
         Replace the current initialization file contents with the
-        'content' string or remove if None. `restart`, which defaults
-        to true, specifies whether any running shared back-ends should
-        be shut down, so that subsequent requests will be handled by
-        back-ends that include the new code.
+        ``content`` string or remove if None.
+
+        :param content: New init file conent.
+        :type content: string
+        :param restart: specifies whether any running shared back-ends should
+                        be shut down, so that subsequent requests will be handled by
+                        back-ends that include the new code. Default is ``True``.
         """
         return self._client.setInitfile(content, restart)
 
     def openSession(self, spec, autocommit=False, lifetime=None, loadinitfile=False):
         """
         Open a session on a federated, reasoning, or filtered store.
-        Use the helper functions in the franz.openrdf.sail.spec module
+
+        Use the helper functions in the :mod:`franz.openrdf.sail.spec` module
         to create the spec string.
+
+        :param spec: Session :mod:`specification string <franz.openrdf.sail.spec>`.
+        :type spec: string
+        :param autocommit: Autocommit mode (default: ``False`` = start a transaction).
+        :type autocommit: bool
+        :param lifetime: The number of seconds a session can be idle before being terminated.
+                         The default value is 300 (5 minutes). The maximum acceptable value
+                         is 21600 (6 hours).
+        :param loadinitfile: If ``True`` the init file will be loaded into the new session.
+                             The default is False.
+        :type loadinitfile: bool
+        :return: A connection to the new session.
+        :rtype: RepositoryConnection
         """
         minirep = self._client.openSession(spec, autocommit=autocommit, lifetime=lifetime, loadinitfile=loadinitfile)
         return RepositoryConnection(Repository(None, None, minirep))
 
     def listScripts(self):
         """
-        List the registered scripts.
+        Return the list of Sitescripts currently on the server.
+
+        When a user creates a session, they can choose to load one or more
+        of these scripts into the session.
+
+        :return: A list of script names.
+        :rtype: list[string]
         """
         return self._client.listScripts()
 
     def addScript(self, module, code):
+        """
+        Create or replace a sitescript.
+
+        :param module: Script name.
+        :type module: string
+        :param code: Script content.
+        :type code: string
+        """
         return self._client.addScript(module, code)
 
     def deleteScript(self, module):
+        """
+        Delete a sitescript.
+        :param module: Script name.
+        :type module: string
+        """
         return self._client.deleteScript(module)
 
     def getScript(self, module):
+        """
+        Get the body of a sitescript.
+        :param module: Script name.
+        :type module: string
+        :return: Script content.
+        :rtype: string
+        """
         return self._client.getScript(module)
 
     def openFederated(self, repositories, autocommit=False, lifetime=None, loadinitfile=False):
@@ -200,6 +276,19 @@ class AllegroGraphServer(object):
         designators, which can be Repository or RepositoryConnection
         objects, strings (naming a store in the root catalog, or the
         URL of a store), or (storename, catalogname) tuples.
+
+        :param repositories: List of repositories to federate.
+        :type repositories: list[string|(string, string)|Repository|RepositoryConnection]
+        :param autocommit: Autocommit mode (default: ``False`` = start a transaction).
+        :type autocommit: bool
+        :param lifetime: The number of seconds a session can be idle before being terminated.
+                         The default value is 300 (5 minutes). The maximum acceptable value
+                         is 21600 (6 hours).
+        :param loadinitfile: If ``True`` the init file will be loaded into the new session.
+                             The default is False.
+        :type loadinitfile: bool
+        :return: A connection to the new session.
+        :rtype: RepositoryConnection
         """
         def asRepoString(x):
             if isinstance(x, basestring): return spec.local(x)
@@ -249,19 +338,19 @@ class AllegroGraphServer(object):
 
     def addUserAccess(self, name, read, write, catalog='*', repository='*'):
         """
-        This is used to grant read/write access to a user. It takes four parameters:
+        Grant read/write access to a user.
 
-        read
-                Whether to grant read access. A boolean, defaults to false.
-        write
-                Whether to grant write access. Boolean, defaults to false.
-        catalog
-                Which catalog to grant the access on. Leave off or pass * to grant
-                access on all catalogs. Again, use / for the root catalog.
-        repository
-                Specifies the repository that access is granted on. Passing *,
-                or leaving the parameter off, means all repositories in the
-                given catalog.
+        :param read: Whether to grant read access. Defaults to ``False``.
+        :type read: bool
+        :param write: Whether to grant write access. Defaults to ``False``.
+        :type write: bool
+        :param catalog: Which catalog to grant the access on. Leave off or pass ``"*"`` to grant
+                        access on all catalogs. Use ``"/"`` for the root catalog.
+        :type catalog: string
+        :param repository: Specifies the repository that access is granted on. Passing ``"*"``,
+                           or leaving the parameter off, means all repositories in the
+                           given catalog.
+        :type repository: string
         """
         self._client.addUserAccess(name, read, write, catalog, repository)
 
@@ -468,11 +557,32 @@ class Catalog(object):
         return self.mini_catalog.listRepositories()
 
     def deleteRepository(self, name):
+        """
+        Delete a repository.
+        :param name: Repository name.
+        :type name: string
+        """
         return self.mini_catalog.deleteRepository(name)
 
     def getRepository(self, name, access_verb):
         """
-        Create a mini-repository and execute a RENEW, OPEN, CREATE, or ACCESS.
+        Creates or opens a repository.
+        
+        :param name: Repository name.
+        :type name: string
+        :param access_verb: Determines mode of operation. Possible values are:
+
+                                - **Repository.RENEW** clears the contents of an existing
+                                   repository before opening. If the indicated repository does not
+                                   exist, it creates one.
+                                -  **Repository.OPEN** opens an existing repository, or throws an
+                                   exception if the repository is not found.
+                                -  **Repository.ACCESS** opens an existing repository, or creates a
+                                   new one if the repository is not found.
+                                -  **Repository.CREATE** creates a new repository, or throws an
+                                   exception if one by that name already exists.
+        :return: A repository object.
+        :rtype: Repository
         """
         access_verb = access_verb.upper()
         name = urllib.parse.quote_plus(name)
@@ -504,13 +614,14 @@ class Catalog(object):
 
     def createRepository(self, name, indices=None):
         """
-        createsRepository - makes a new repository with the given name.
+        Creates a new repository with the given name.
 
-        indices - if provided, creates a store with the given indices
-        deleteDuplicates - sets behavior for duplicate removal. See
-            http protocol documentation for description. None will
-            result in using the server's default behavior.
+        :param name: Repository name.
+        :type name: string
+        :param indices: If provided, creates a store with the given indices.
+                        (e.g. ``["spogi, "gspoi", ...]``)
+        :type indices: list[string]
+        :return: A repository object.
+        :rtype: Repository
         """
         return Repository(self, name, self.mini_catalog.createRepository(name, indices=indices))
-
-
